from django.template.response import TemplateResponse
from django.core.urlresolvers import reverse
from django.shortcuts import get_object_or_404
from django.utils.translation import ugettext as _
from django.utils.importlib import import_module
from django.utils.http import urlencode
from django.conf import settings
from django.db import transaction
from django.contrib import messages
from django.contrib.auth import login as auth_login, logout as auth_logout
from django.http import HttpResponseRedirect
from bambu.signup.models import *

SIGNUP_FORM = getattr(settings, 'SIGNUP_FORM', 'bambu.signup.forms.RegistrationForm')
LOGIN_FORM = getattr(settings, 'LOGIN_FORM', 'bambu.signup.forms.AuthenticationForm')
LOGIN_URL = getattr(settings, 'LOGIN_URL')
VALIDATION_FORM = getattr(settings, 'SIGNUP_VALIDATION_FORM', 'bambu.signup.forms.EmailValidationForm')
PASSWORD_RESET_FORM = getattr(settings, 'PASSWORD_RESET_FORM', 'bambu.signup.forms.PasswordResetForm')
LOGIN_REDIRECT_URL = getattr(settings, 'LOGIN_REDIRECT_URL', '/')
LOGOUT_REDIRECT_URL = getattr(settings, 'LOGOUT_REDIRECT_URL', '/')

def _get_form(constant):
	module, dot, klass = constant.rpartition('.')
	module = import_module(module)
	return getattr(module, klass)

def register(request):
	if request.user.is_authenticated():
		messages.warning(request, u'You already have an account.')
		return HttpResponseRedirect(LOGIN_REDIRECT_URL)
	
	form_class = _get_form(SIGNUP_FORM)
	form = form_class(data = request.POST or None)
	next = request.POST.get('next',
		request.GET.get('next',
			LOGIN_REDIRECT_URL
		)
	)
	
	if request.method == 'POST' and form.is_valid():
		user = form.save(next = next)
		auth_login(request, user)
		
		return HttpResponseRedirect(
			reverse('signup_complete')
		)
	
	return TemplateResponse(
		request,
		'signup/register.html',
		{
			'form': form,
			'next': next
		}
	)

def register_complete(request):
	return TemplateResponse(
		request,
		'signup/complete.html',
		{
			'next': next
		}
	)

def login(request):
	form_class = _get_form(LOGIN_FORM)
	form = form_class(request.POST or None)
	
	if request.method == 'POST' and form.is_valid():
		if form.login(request):
			return HttpResponseRedirect(
				request.GET.get('next', LOGIN_REDIRECT_URL)
			)
	
	return TemplateResponse(
		request,
		'signup/login.html',
		{
			'form': form
		}
	)
	
def logout(request):
	auth_logout(request)
	messages.success(request, u'You have been logged out.')
	return HttpResponseRedirect(LOGOUT_REDIRECT_URL)

@transaction.commit_on_success
def verify_email(request, guid):
	validation = get_object_or_404(EmailValidation, guid = guid)
	next = validation.next_url
	form_class = _get_form(VALIDATION_FORM)
	form = form_class(data = request.POST or None, instance = validation)
	
	if request.method == 'POST' and form.is_valid():
		validation.user.is_active = True
		validation.user.save()
		validation.delete()
		messages.success(request,
			_('Thanks for confirming your email address.')
		)
		
		return HttpResponseRedirect(
			'%s?%s' % (
				LOGIN_URL,
				urlencode(
					{
						'next': next or LOGIN_REDIRECT_URL
					}
				)
			)
		)
	
	return TemplateResponse(
		request,
		'signup/validation.html',
		{
			'form': form
		}
	)

def reset_password(request, guid = None):
	next = request.POST.get('next', request.GET.get('next'))
	
	if guid:
		reset = get_object_or_404(PasswordReset, guid = guid)
		reset.reset()
		
		messages.success(request,
			_('Your new password should be in your inbox shortly.')
		)
		
		return HttpResponseRedirect(settings.LOGIN_URL)
	
	form_class = _get_form(PASSWORD_RESET_FORM)
	form = form_class(request.POST or None)
	
	if request.method == 'POST' and form.is_valid():
		with transaction.commit_on_success():
			try:
				user = User.objects.get(email__iexact = form.cleaned_data['email'])
				if not user.password_resets.exists():
					reset = user.password_resets.create(
						next_url = next
					)
			except User.DoesNotExist:
				pass
		
		return TemplateResponse(
			request,
			'signup/password-reset.html'
		)
	
	return TemplateResponse(
		request,
		'signup/forgot-password.html',
		{
			'form': form
		}
	)



# encoding: utf-8

from django import forms
from django.utils.translation import ugettext_lazy as _
from django.contrib.auth.models import User
from django.contrib.auth import authenticate, login
from django.conf import settings
from bambu.signup.models import EmailValidation

class RegistrationForm(forms.Form):
	first_name = forms.CharField(max_length = 20)
	last_name = forms.CharField(max_length = 20)
	email = forms.EmailField(label = _('Email address'))
	
	username = forms.RegexField(
		label = _('Username'), max_length = 30, regex = r'^[\w.@+-]+$',
		help_text = _('Required. 30 characters or fewer. Letters, digits and @/./+/-/_ only.'),
		error_messages = {
			'invalid': _('This value may contain only letters, numbers and @/./+/-/_ characters.')
		},
		widget = forms.TextInput(
			attrs = {
				'autocomplete': 'off'
			}
		)
	)
	
	password1 = forms.CharField(
		label = _('Password'),
		widget = forms.PasswordInput(
			attrs = {
				'autocomplete': 'off'
			}
		)
	)
	
	password2 = forms.CharField(label = _('Confirm password'),
		widget = forms.PasswordInput(
			attrs = {
				'autocomplete': 'off'
			}
		),
		help_text = _('Enter the same password as above, for verification.')
	)
	
	def clean_username(self):
		username = self.cleaned_data['username']
		
		try:
			User.objects.get(username__iexact = username)
		except User.DoesNotExist:
			return username.lower()
		
		raise forms.ValidationError(
			_('A user with that username already exists.')
		)
	
	def clean_password2(self):
		password1 = self.cleaned_data.get('password1', '')
		password2 = self.cleaned_data['password2']
		
		if password1 != password2:
			raise forms.ValidationError(
				_('The two password fields don\'t match.')
			)
		
		return password2
	
	def clean_email(self):
		email = self.cleaned_data['email']

		try:
			User.objects.get(email__iexact = email)
		except User.DoesNotExist:
			return email
		
		raise forms.ValidationError(
			_('A user with that email address already exists.')
		)
	
	def save(self, commit = True, next = None):
		user = User.objects.create_user(
			username = self.cleaned_data['username'],
			password = self.cleaned_data['password1'],
			email = self.cleaned_data['email']
		)
		
		user.first_name = self.cleaned_data['first_name']
		user.last_name = self.cleaned_data['last_name']
		user.save()
		user.email_validations.create(next_url = next)
		
		return authenticate(
			username = self.cleaned_data['username'],
			password = self.cleaned_data['password1']
		)

class AuthenticationForm(forms.Form):
	username = forms.RegexField(
		label = _('Username'), max_length = 30, regex = r'^[\w.@+-]+$'
	)
	
	password = forms.CharField(label = _('Password'), widget = forms.PasswordInput)
	
	def clean(self):
		self._user_cache = authenticate(
			username = self.cleaned_data['username'],
			password = self.cleaned_data['password']
		)
		
		if not self._user_cache:
			raise forms.ValidationError('Please enter the correct username and password.')
		
		return self.cleaned_data
	
	def login(self, request):
		login(request, self._user_cache)
		return self._user_cache

class PasswordResetForm(forms.Form):
	email = forms.EmailField(label = _('Email address'))

class EmailValidationForm(forms.Form):
	code = forms.CharField(max_length = 10, label = u'Verification code')
	
	def __init__(self, *args, **kwargs):
		self.instance = kwargs.pop('instance')
		super(EmailValidationForm, self).__init__(*args, **kwargs)
	
	def clean_code(self):
		if self.cleaned_data.get('code') != self.instance.code:
			raise forms.ValidationError('That code is not correct.')


from django.conf.urls import patterns, url
from bambu.signup.views import *
from bambu.signup.views import ajax
from django.conf import settings

if 'bambu.bootstrap' in settings.INSTALLED_APPS or 'bambu.bootstrap.v2' in settings.INSTALLED_APPS:
	from bambu.bootstrap.v2.decorators import body_classes
elif 'bambu.bootstrap.v3' in settings.INSTALLED_APPS:
	from bambu.bootstrap.v3.decorators import body_classes
else:
	def body_classes(view, *classes):
		return view

urlpatterns = patterns('',
	url(r'^signup/$', body_classes(register, 'signup'), name = 'signup'),
	url(r'^signup/complete/$',
		body_classes(register_complete, 'signup', 'signup-complete'), name = 'signup_complete'
	),
	url(r'^login/$', login, name = 'login'),
	url(r'^logout/$', logout, name = 'logout'),
	url(r'^verify/(?P<guid>[a-f0-9]{8}-[a-f0-9]{4}-[a-f0-9]{4}-[a-f0-9]{4}-[a-f0-9]{12})/$',
		body_classes(verify_email, 'signup-verify'), name = 'verify_email'
	),
	url(r'^reset/$', body_classes(reset_password, 'signup-reset'), name = 'forgot_password'),
	url(r'^reset/(?P<guid>[a-f0-9]{8}-[a-f0-9]{4}-[a-f0-9]{4}-[a-f0-9]{4}-[a-f0-9]{12})/$',
		body_classes(reset_password, 'signup-reset', 'signup-reset-result'), name = 'reset_password'
	),
	url(r'^ajax/username-exists\.js$', ajax.username_exists, name = 'username_exists_ajax'),
	url(r'^ajax/email-exists\.js$', ajax.email_exists, name = 'email_exists_ajax')
)


from django.contrib.auth.models import User
from django.http import HttpResponse

def _ajax_view(request, field):
	value = request.GET.get('v')
	callback = request.GET.get('callback')
	
	if not value:
		response = 'false'
	else:
		response = User.objects.filter(
			**{
				'%s__iexact' % field: value
			}
		).exists() and 'true' or 'false'
	
	if callback:
		response = '%s(%s)' % (callback, response)
	
	return HttpResponse(response, mimetype = 'application/json')

def username_exists(request):
	return _ajax_view(request, 'username')

def email_exists(request):
	return _ajax_view(request, 'email')


from django.db import models
from django.contrib.auth.models import User
from bambu.mail import render_to_mail
from uuid import uuid4
import random, string

class EmailValidation(models.Model):
	user = models.ForeignKey('auth.User', related_name = 'email_validations', unique = True)
	next_url = models.CharField(max_length = 200, null = True, blank = True)
	guid = models.CharField(max_length = 36, unique = True)
	code = models.CharField(max_length = 10)
	sent = models.DateTimeField(auto_now_add = True)
	
	def __unicode__(self):
		return self.guid
	
	def save(self, *args, **kwargs):
		if not self.guid:
			self.guid = unicode(uuid4())
		
		if not self.code:
			self.code = ''.join(
				random.sample(string.letters + string.digits, 10)
			).upper()
		
		new = not self.pk
		super(EmailValidation, self).save(*args, **kwargs)
		
		self.user.is_active = False
		self.user.save()
		
		if new:
			render_to_mail(
				subject = u'Please confirm your email address',
				template = 'signup/mail/validate.txt',
				context = {
					'name': self.user.first_name or self.user.username,
					'guid': self.guid,
					'code': self.code,
					'next': self.next_url
				},
				recipient = self.user
			)
	
	class Meta:
		ordering = ('-sent',)
		get_latest_by = 'sent'

class PasswordReset(models.Model):
	user = models.ForeignKey(User, related_name = 'password_resets', unique = True)
	guid = models.CharField(max_length = 36, unique = True)
	next_url = models.CharField(max_length = 200, null = True, blank = True)
	sent = models.DateTimeField(auto_now_add = True)
	
	def __unicode__(self):
		return self.guid
	
	def save(self, *args, **kwargs):
		if not self.guid:
			self.guid = unicode(uuid4())
		
		new = not self.pk
		super(PasswordReset, self).save(*args, **kwargs)
		
		if new:
			render_to_mail(
				subject = u'Forgotten your password?',
				template = 'signup/mail/password-reset.txt',
				context = {
					'name': self.user.first_name or self.user.username,
					'guid': self.guid,
					'next': self.next_url
				},
				recipient = self.user
			)
	
	def reset(self):
		password = User.objects.make_random_password(10)
		
		self.user.set_password(password)
		self.user.save()
		self.delete()
		
		render_to_mail(
			subject = u'Your new password',
			template = 'signup/mail/password.txt',
			context = {
				'name': self.user.first_name or self.user.username,
				'username': self.user.username,
				'password': password,
				'next': self.next_url
			},
			recipient = self.user
		)
	
	class Meta:
		ordering = ('-sent',)
		get_latest_by = 'sent'